/*
 *  freebooters.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <cstdlib>
#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL_ttf.h>
#include <SDL_image.h>
#include <sys/types.h>
#include <time.h>

#include <sstream>
#include "menu.h"
#include "world.h"
#include "tools.h"
#include "gamemenus.h"
#include "sdl_setup.h"
#include "harbour.h"
#include "player.h"
#include "gameplay.h"
#include "tools.h"
#include "readdata.h"

using namespace std;

int main(int argc, char *argv[])
{
   Tools t;

   srand(time(0));
   
   Properties properties;

   SDL_Setup setup;
   properties.screen = setup.sdl();

   properties.font = t.load_ttf(30, "vera.ttf");
   properties.small_font = t.load_ttf(14, "vera.ttf");

   Uint32 col = SDL_MapRGB(properties.screen->format, 255,0,0);
   SDL_GetRGB(col, properties.screen->format, &properties.on_color.r, &properties.on_color.g, &properties.on_color.b);

   col = SDL_MapRGB(properties.screen->format, 255,255,255);
   SDL_GetRGB(col, properties.screen->format, &properties.off_color.r, &properties.off_color.g, &properties.off_color.b);

   col = SDL_MapRGB(properties.screen->format, 100, 100, 100);
   SDL_GetRGB(col, properties.screen->format, &properties.dark_color.r, &properties.dark_color.g, &properties.dark_color.b);
   
   col = SDL_MapRGB(properties.screen->format, 20,66,95);
   SDL_Color bg_color;
   SDL_GetRGB(col, properties.screen->format, &bg_color.r, &bg_color.g, &bg_color.b);

   properties.background = "freebooters.jpg";

   World world;
   world.set_state(GS_STARTMENU);

   while (world.get_state() != GS_GAMEOVER)
   {
      
      Gamemenus menus(&properties);
      menus.set_background("freebooters.jpg");
      Gameplay play;

      menuselect startchoice = menus.men_start();

      if (startchoice.selected == "New game")
      {
         world.remove_players();
         world.remove_opponent(0);
         string diff = "DIFF_MEDIUM";
         menuselect newselect = menus.men_new_player(world.get_harbours());
         if (newselect.strs["Difficulty"] == "Boring")
            diff = "DIFF_EASY";
         if (newselect.strs["Difficulty"] == "Challenging")
            diff = "DIFF_MEDIUM";
         if (newselect.strs["Difficulty"] == "Impossible")
            diff = "DIFF_HARD";
         
         string nat;
         if (newselect.strs["Nationality"] == "Spanish")
            nat = "sp";
         
         if (newselect.strs["Nationality"] == "Dutch")
            nat = "du";
         
         if (newselect.strs["Nationality"] == "French")
            nat = "fr";
         
         if (newselect.strs["Nationality"] == "English")
            nat = "en";
         
         world.add_player(Player(newselect.strs["Player name"], nat, diff,
                                 world.get_harbour_by_name(newselect.strs["Start Harbour"]),
                                 newselect.strs["Advantage"],newselect.strs["Color"]));

         world.set_state(GS_RUNNING);

         world.add_nation("du");
         world.add_nation("en");
         world.add_nation("fr");
         world.add_nation("sp");
      }
      
      if (startchoice.selected == "Load game")
      {
         string savegamename;
         savegamename = menus.men_opt_loadgame(&world);
         
         if (savegamename != "Back")
            world.load_game(savegamename, "Unser erstes Test-Savegame");
      }

      if (startchoice.selected == "Quit game")
      {
         exit(0);
      }

      while (world.get_state() == GS_RUNNING)
      {
         play.turn(&world, &menus);
         world.turn();
      }
   }

   setup.cleanup();
   return 0;
}











































