/*
 *  gameplay.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gameplay.h"
#include "gamemenus.h"
#include <sstream>
#include "tools.h"
#include "seabattle.h"

using namespace std;

Gameplay::Gameplay()
{
   Tools t;
   gameplay_parameters = readdata(t.data_path("gameplay-parameters.conf"));
}

void Gameplay::turn(World* world, Gamemenus* m)
{
   Tools t;

   for (int i=0; i< world->get_nr_players(); i++)
   {
      Player* pl = world->get_player(i);

      pl->set_round(world->get_turn());
      
      if (pl->get_event() == GE_SEABATTLE)
      {
         pl->set_event(GE_NONE);
         world->remove_opponent(0);
         world->create_opponent();

         Player* op = world->get_opponent(0);

         string encounter = m->men_ship_encounter(pl, op);

         if (encounter =="Attack")
         {
            if (pl->opponent_is_willing_to_fight(op, world))
               fight(pl, op, m, world);
            
            else  // the opponent is trying to escape
            {   
               if (pl->opponent_could_escape(op, world))
                  m->msg_box (pl, "Your opponent could escape", "on-sea.jpg");
               else
                  fight(pl, op, m, world);
            }
         }
            
         if (encounter =="Sail away")
         {
            if (pl->opponent_is_willing_to_fight(op, world))
            {   
               if (pl->escape_from_enemy(op, world))
                  m->msg_box (pl, "You could escape", "on-sea.jpg");
               else
               {
                  m->msg_box (pl, "The other ship attacks", "on-sea.jpg");
                  fight(pl, op, m, world);
               }
            }
         }

         if (world->get_state() == GS_STARTMENU)
            return;
         
         world->remove_opponent(0);
         pl->set_state("PS_ONSEA");
      }
      
      if (pl->get_event() == GE_STARVE)
      {
         int losses = (rand() % ((int)(pl->get_ship()->get_crew_members() * 0.12)));
//                                 gameplay_parameters.get_int("rate_of_dying_crew_members_when_starving"))));
         ostringstream ost;
         ost <<  losses << " men died.";
         m->msg_box (pl, "Your crew is starving.", ost.str(), "on-sea.jpg");
         pl->get_ship()->add_crew_members(-losses, 0);
      }

      if (pl->get_event() == GE_SCURVY)
      {
         int losses = (rand() % ((int)(pl->get_ship()->get_crew_members() *
                                 gameplay_parameters.get_int("rate_of_dying_crew_members_when_scurvy"))));
         ostringstream ost;
         ost << losses << " men died.";
         m->msg_box (pl, "The scurvy broke out", ost.str(), "on-sea.jpg");
         pl->get_ship()->add_crew_members(-losses, 0);
         pl->set_event(GE_NONE);
      }

      if (pl->get_event() == GE_MUTINY)
      {
         string mutinyselect = m->men_mutiny_select(pl);

         bool lost_leadership = true;
         bool killed_by_crew = false;

         // GAMEPLAYFIXME, create a formula that includes the player's skills in ferocity, fencing etc.
         // FIXME, parametrize me through gameplay-parameters.conf
         if (mutinyselect == "Careen the leaders")
         {
            if (rand() % 100 > 5)
               lost_leadership = false;
            else
               killed_by_crew = true;
         }

         if (mutinyselect == "Bribe them")
         {
            // GAMEPLAYFIXME, adapt towards crew size and satisfaction
            // is should also be configurable how much money shall spent
            // to satisfy the crew
            if (pl->get_gold() < gameplay_parameters.get_int("amount_of_gold_to_satisfy_crew_on_mutiny"))
               ;
            else
            {
               pl->add_gold(-gameplay_parameters.get_int("amount_of_gold_to_satisfy_crew_on_mutiny"));
               lost_leadership = true;
            }
         }

         if (lost_leadership == false)
         {
            m->msg_box (pl, "You managed to control", "the situation", "on-sea.jpg");
            pl->get_ship()->survived_mutiny();
            pl->set_event(GE_NONE);
         }
         
         else if (lost_leadership == true && killed_by_crew == false)
         {
            m->msg_box (pl, "You crew dropped you off", "a lonely island", "on-sea.jpg");
            m->men_player_dead(pl);
            world->set_state(GS_STARTMENU);
            return;
         }

         else if (lost_leadership == true && killed_by_crew == true)
         {
            m->msg_box (pl, "Your crew became angry and", "killed you", "on-sea.jpg");
            m->men_player_dead(pl);
            world->set_state(GS_STARTMENU);
            return;
         }

      }

      if (pl->get_event() == GE_STORM)
      {
         bool damaged = false;
         bool lost = false;

         if (t.rnd_percent(gameplay_parameters.get_int("probability_that_ship_is_damaged_when_getting_in_a_storm")))
            damaged = true;

         if (t.rnd_percent(gameplay_parameters.get_int("probability_that_crew_is_lost_when_getting_in_a_storm")))
            lost = true;

         ostringstream ost;
         int losses;

         if (lost)
         {
            losses = (rand() % (int)(pl->get_ship()->get_crew_members() *
                                     gameplay_parameters.get_float("maximum_of_crew_members_overboard")));
            ost << losses << " men went overboard.";
            pl->get_ship()->add_crew_members(-losses, 0);
         }
         
         if (damaged)
         {
            ost << "Your ship is damaged.";
            pl->get_ship()->add_storm_damage();
         }

         m->msg_box (pl, "You run into a storm", ost.str(), "on-sea.jpg");
         pl->set_event(GE_NONE);

         if (pl->get_ship()->get_state() < 0)
         {
            m->msg_box (pl, "Your ship sunk", "on-sea.jpg");
            world->set_state(GS_STARTMENU);
            return;
         }
      }
   }

   if (world->get_state() == GS_STARTMENU)
      return;

   for (int i=0; i< world->get_nr_players(); i++)
   {
      Player* pl = world->get_player(i);
      
      if (pl->get_state() == "PS_ONSEA")
      {
         string seaselect;
         m->set_background("on-sea.jpg");
         while (seaselect != "End turn")
         {
            seaselect = m->men_onsea(pl, world);
            if (seaselect == "View status information")
               view_status(pl, m);

            if (seaselect == "Edit ship settings")
               ship_settings(pl, m);

            if (seaselect == "Game options")
            {
               if (options(world, m))
                  return;
               m->set_background("on-sea.jpg");
            }
         }
         
         if (pl->get_remaining_distance() <= 0)
         {
            pl->set_state("PS_HARBOUR");
            pl->set_target_harbour();
         }
      }

      if (pl->get_state() == "PS_HARBOUR")
      {
         while (pl->get_state() == "PS_HARBOUR")
         {
            string harbselect = m->men_harbour(pl, world);
            if (harbselect == "Visit the market")
            {
               m->set_background("market.jpg");
               string marketselect;

               while (marketselect != "Back")
               {
                  marketselect = m->men_market(pl);
                  if (marketselect == "Buy goods")
                  {
                     string selected_good = m->men_select_goods(pl, pl->get_harbour_pos()->get_market()->get_goods());

                     int bought = m->men_buy_menu(pl, selected_good);

                     if ((bought*pl->get_harbour_pos()->get_market()->get_price(selected_good)) > pl->get_gold())
                        m->msg_box (pl, "You don't have enough gold", "market.jpg");
                     else if (bought > pl->get_harbour_pos()->get_market()->get_stock(selected_good))
                        m->msg_box (pl, "Your request exceeds our stock", "market.jpg");
                     else if (bought < 0)
                        ;
                     else if (bought > pl->get_ship()->get_cargo_free())
                        m->msg_box (pl, "You don't have enough cargo capacity", "market.jpg");
                     else 
                     {
                        float charm2 = gameplay_parameters.get_float("excellent_charm_trade_reduction_factor");
                        float charm1 = gameplay_parameters.get_float("good_charm_trade_reduction_factor");

                        if (pl->get_charme() == 2)
                           pl->add_gold(-(int)((1-charm2)*(pl->get_harbour_pos()->get_market()->buy(bought, selected_good,
                                                                                        pl->get_ship()))));
                        if (pl->get_charme() == 1)
                           pl->add_gold(-(int)((1-charm1)*(pl->get_harbour_pos()->get_market()->buy(bought, selected_good,
                                                                                        pl->get_ship()))));
                        if (pl->get_charme() == 0)
                           pl->add_gold(-(1*(pl->get_harbour_pos()->get_market()->buy(bought, selected_good,
                                                                                      pl->get_ship()))));
                        pl->set_event(GE_NONE);
                        pl->get_ship()->feed_hungry_mouths();
                     }
            
                  }
                  
                  if (marketselect == "Sell goods")
                  {
                     string selected_good = m->men_select_goods(pl, pl->get_ship()->get_goods());
                     int sold = m->men_sell_menu(pl, selected_good);
                     
                     if (sold > pl->get_ship()->get_stock(selected_good))
                        m->msg_box (pl, "Your request exceeds your stock", "market.jpg");
                     else if (sold < 0)
                        m->msg_box (pl, "Don't try to fool me", "market.jpg");
                     else
                     {
                        float charm2 = gameplay_parameters.get_float("excellent_charm_trade_reduction_factor");
                        float charm1 = gameplay_parameters.get_float("good_charm_trade_reduction_factor");

                        if (pl->get_charme() == 2)
                           pl->add_gold((int)(1+charm2)*(pl->get_harbour_pos()->get_market()->sell(sold, selected_good, pl->get_ship())));

                        if (pl->get_charme() == 1)
                           pl->add_gold((int)(1+charm1)*(pl->get_harbour_pos()->get_market()->sell(sold, selected_good, pl->get_ship())));

                        if (pl->get_charme() == 0)
                           pl->add_gold(pl->get_harbour_pos()->get_market()->sell(sold, selected_good, pl->get_ship()));
                     }
                     
                  }

                  if (marketselect == "Prices")
                     m->men_market_prices(pl);

                  if (marketselect == "Refresh provisions")
                  {
                     int bought = m->men_buy_food(pl);

                     if ((bought*pl->get_harbour_pos()->get_market()->get_price("Food")) > pl->get_gold())
                        m->msg_box (pl, "You don't have enough gold", "market.jpg");
                     else if (bought > pl->get_harbour_pos()->get_market()->get_stock("Food"))
                        m->msg_box (pl, "Your request exceeds our stock", "market.jpg");
                     else if (bought < 0)
                        ;
                     else if (bought > pl->get_ship()->get_food_free())
                        m->msg_box (pl, "You don't have enough cargo capacity", "market.jpg");
                     else 
                     {
                        float charm2 = gameplay_parameters.get_float("excellent_charm_trade_reduction_factor");
                        float charm1 = gameplay_parameters.get_float("good_charm_trade_reduction_factor");

                        if (pl->get_charme() == 2)
                           pl->add_gold(-(int)((1-charm2)*(pl->get_harbour_pos()->get_market()->buy_food(bought, pl->get_ship()))));
                        if (pl->get_charme() == 1)
                           pl->add_gold(-(int)((1-charm1)*(pl->get_harbour_pos()->get_market()->buy_food(bought, pl->get_ship()))));
                        if (pl->get_charme() == 0)
                           pl->add_gold(-(1*(pl->get_harbour_pos()->get_market()->buy_food(bought, pl->get_ship()))));
                        pl->set_event(GE_NONE);
                        pl->get_ship()->feed_hungry_mouths();
                     }
                  }
               }
            }

            if (harbselect == "View status information")
               view_status(pl, m);
            
            if (harbselect == "Visit the tavern")
            {
               m->set_background("tavern.jpg");
               string tavernselect;
               while (tavernselect != "Back")
               {
                  tavernselect = m->men_tavern(pl);
                  if (tavernselect == "Hire crew members")
                  {
                     int tohire = m->men_hire_crew(pl);

                     if (tohire <= 0)
                        ;
                     else if (tohire > pl->get_harbour_pos()->get_crew_available())
                        m->msg_box (pl, "Not enough people in the tavern", "tavern.jpg");
                     else
                     {
                        int hired = pl->get_harbour_pos()->hire_men(tohire);
                        int payinc = pl->get_harbour_pos()->get_crew_pay();
                        if (hired != -1)
                           pl->get_ship()->add_crew_members(hired, (payinc*hired));
                     }
                  }
                  
                  if (tavernselect == "Get the latest gossip")
                     m->men_stat_nations(pl, world);
               }
            }

            if (harbselect == "Edit ship settings")
               ship_settings(pl, m);

            if (harbselect == "Game options")
               if (options(world, m))
                  return;

            if (harbselect == "Visit the gouvernor")
            {
               m->set_background("governor.jpg");
               string govselect;
               while (govselect != "Back")
               {
                  govselect = m->men_gov_select(pl);
                  if (govselect == "Negotiate a letter of marque")
                  {
                     if (pl->get_harbour_pos()->get_letter_of_marque(pl))
                     {
                        if (m->bool_box(pl, "Willing to give 50% to the crown?", "shipyard.jpg"))
                           pl->set_freebooting(true);
                        else
                           pl->set_freebooting(false);
                     }
                     else
                        m->msg_box (pl, "You're not a subject of our throne", "shipyard.jpg");
                  }
               }
               
               
            }
            
            if (harbselect == "Visit the shipyard")
            {
               m->set_background("shipyard.jpg");
               string dockselect;
               while (dockselect != "Back")
               {
                  dockselect = m->men_dock(pl);

                  if (dockselect == "Buy a ship" || dockselect == "Upgrade ship")
                  {
                     string newship = m->men_dock_buy_ship(pl);
                     
                     if (newship != "NONE")
                     {
                        int shipprice = pl->get_harbour_pos()->get_ship_price(newship);
                        if (shipprice > pl->get_gold())
                           m->msg_box (pl, "You don't have enough gold", "shipyard.jpg");
                        else
                        {
                           pl->add_gold(-shipprice);
                           string ownship = pl->get_ship()->get_type();
                           string ownflag = pl->get_ship()->get_flag();
                           std::vector<Good> owngoods = pl->get_ship()->get_goods();
                           int valueown = pl->get_harbour_pos()->get_ship_price(ownship);
                           int owncannons = pl->get_ship()->get_cannons();
                           int owncrew = pl->get_ship()->get_crew_members();
                           int ownpay = pl->get_ship()->get_crew_pay();
                           float ownfood = pl->get_ship()->get_food();
                           int shipstate = pl->get_ship()->get_state();
                           pl->add_gold((int)(valueown*gameplay_parameters.get_float("trade_in_price_factor_ship")*
                                              shipstate*0.01));
                           string shipname = m->men_ship_rename(pl);
                           pl->set_ship(newship, shipname, 100);
                           pl->get_ship()->set_flag(ownflag);
                           pl->get_ship()->add_food(ownfood);
                           pl->get_ship()->set_cannons(owncannons);
                           for (vector<Good>::iterator gi = owngoods.begin(); gi != owngoods.end(); gi++)
                              pl->get_ship()->add_good(gi->get_stock(), gi->get_name());
                           pl->get_ship()->recalculate_cargo();
                           pl->get_ship()->add_crew_members(owncrew, ownpay);
                        }
                     }
                  }

                  if (dockselect == "Improve equipment")
                  {
                     string eqselect;
                     while (eqselect != "Back")
                     {
                        eqselect = m->men_dock_upgrade_equipment(pl);
                        if (eqselect == "Buy more cannons")
                        {
                           int bc = m->men_dock_upgrade_cannons(pl);
                           int cannon_costs = pl->get_ship()->add_cannons(bc, gameplay_parameters.get_int("cannon_price"),
                                                                          pl->get_gold());

                           if (cannon_costs != -1)
                              pl->add_gold(-cannon_costs);
                        }
                        
                        if (eqselect == "Improve plating")
                        {
                           int bp = m->men_dock_upgrade_plating(pl);
                           int plating_costs = pl->get_ship()->add_plating(bp, gameplay_parameters.get_int("plating_price"),
                                                                           pl->get_gold());

                           if (plating_costs != -1)
                              pl->add_gold(-plating_costs);
                        }
                     }
                  }

                  if (dockselect == "Repair ship")
                  {
                     int r = m->men_dock_repair(pl);
                     int cost = pl->get_harbour_pos()->get_ship_price(pl->get_ship()->get_type());
                     int repair_costs = pl->get_ship()->repair(r, cost, pl->get_gold());
                     
                     if (repair_costs != -1)
                        pl->add_gold(-repair_costs);
                  }
         
               }

            }

            if (harbselect == "End turn")
            {
               pl->stay_in_harbour();
               break;
            }

            if (harbselect == "Sail away")
            {
               if (!pl->has_ship())
                  m->msg_box (pl, "You don't have a ship", "on-sea.jpg");
               else if (!pl->get_ship()->enough_crew())
                  m->msg_box (pl, "You don't have enough deck hands", "on-sea.jpg");
               else if (!pl->get_ship()->enough_state()) 
                  m->msg_box (pl, "Your ship is too broken", "on-sea.jpg");
               else
               {
                  Harbour* target = m->men_set_sail(pl->get_harbour_pos(), world);

                  if (target != 0)
                  {
                     pl->sail_to(target, world);
                     pl->set_state("PS_ONSEA");
                  }
               }
            }
            
         }
      }
   }

   if (world->get_state() == GS_GAMEOVER)
      exit(0);

}

// returns true if a game has been loaded
bool Gameplay::options(World* world, Gamemenus* m)
{
   m->set_background("freebooters.jpg");

   string option;
   while (option != "Back")
   {
       option = m->men_game_options(world);
   
       if (option == "Load game")
       {
          string load = m->men_opt_loadgame(world);

          if (load != "Back")
          {
             world->load_game(load, "Unser erstes Test-Savegame");

             return true; // Game has been loaded, rebuild game state
          }
       }
       
       if (option == "Save game")
       {
          string save = m->men_opt_savegame(world);
          
          if (save != "Back")
          {
             string description = m->men_opt_savegame_name();
             if (description != "")
                world->save_game(save, description);
          }
       }
       
                
       if (option == "Quit game")
          exit(0);
         
       if (option == "Back")
          ;
   }
   return false;
}

void Gameplay::view_status(Player* pl, Gamemenus* m)
{
   string statselect;
   while (statselect != "Back")
   {
      statselect = m->men_stat_select(pl);
      if (statselect == "View player status")
         m->men_stat_player(pl);
      
      if (statselect == "View ship status")
         m->men_stat_ship(pl);
      
      if (statselect == "View cargo status")
         m->men_stat_cargo(pl);
   }
}


void Gameplay::ship_settings(Player* pl, Gamemenus* m)
{
   string settingselect;
   while (settingselect != "Back")
   {
      settingselect = m->men_settings_ship(pl);
      if (settingselect == "Adjust crew pay")
      {
         int adjust = m->men_settings_crewpay(pl);
         
         if (adjust < 0)
            m->msg_box (pl, "You can't rob them", "shipyard.jpg");
         else if (adjust < 0)
            m->msg_box (pl, "You're too generous", "shipyard.jpg");
         else
            pl->get_ship()->set_pay_percent(adjust);
      }
      
      if (settingselect == "Rename ship")
      {
         string shipname = m->men_ship_rename(pl);
         pl->get_ship()->set_name(shipname);
      }

      if (settingselect == "Change the flag")
      {
         if (pl->get_ship()->get_flag() == "pi")
         {
            if (m->bool_box(pl, "Set the royal flag?", "shipyard.jpg"))
               pl->get_ship()->set_flag(pl->get_nationality());
         }
         else
         {
            if (m->bool_box(pl, "Set the Jolly Roger?", "shipyard.jpg"))
               pl->get_ship()->set_flag("pi");
         }
      }
   }
   
}


void Gameplay::fight(Player* player, Player* opponent, Gamemenus* m, World* world)
{
   m->set_background("on-sea.jpg");
   m->men_fight_prepare(player, opponent);

   Seabattle battle(m->get_screen(), player, opponent);
   int retfight =  battle.run();

   // the AI opponent has been killed
   if (retfight == 1)
   {
      player->add_points(opponent->get_ship()->get_type());
      m->msg_box (player, "You countersunk your opponent", "on-sea.jpg");
   }

   // You have been killed
   if (retfight == 2)
   {
      m->msg_box (player, "Your ship sunk and you drowned.", "on-sea.jpg");
      m->men_player_dead(player);
      world->set_state(GS_STARTMENU);
      return;
   }
      
   // You or the other player boarded
   if (retfight == 3)
   {
      bool board = player->board_ship(opponent);
      m->men_fight_board(player, opponent, board);

      if (board)
      {
         player->add_points(opponent->get_ship()->get_type());
         player->add_gold(opponent->get_gold());
         player->get_ship()->add_food(opponent->get_ship()->get_food());
         string fightselect;
         while (fightselect != "Let them sail away" && fightselect != "Scuttle the ship")
         {
            fightselect = m->men_fight_won(player, opponent);
            if (fightselect == "Loot the ship")
            {
               string selected_good = m->men_select_goods(player, opponent->get_ship()->get_goods());
            
               int looted = m->men_fight_loot(player, selected_good);
                     
               if (looted > opponent->get_ship()->get_stock(selected_good))
               {
                  string msg = "The " + opponent->get_ship()->get_name() + " doesn't";
                  m->msg_box (player, msg, "carry that much", "market.jpg");
               }
               else if (looted < 0)
                  ;
               else if (looted > player->get_ship()->get_cargo_free())
                  m->msg_box (player, "You don't have enough cargo capacity", "on-sea.jpg");
               else 
               {
                  player->get_ship()->add_good(looted, selected_good);
                  opponent->get_ship()->add_good(-looted, selected_good);
               }
            }
         
            if (fightselect == "Scuttle the ship")
            {
               m->msg_box (player, "You scuttle the ship and", "kill the captives", "on-sea.jpg");
               player->raise_ferocity(1);
            }
         
            if (fightselect == "Let them sail away")
            {
               m->msg_box (player, "You spare their lives and they", "sail on", "on-sea.jpg");
               player->raise_ferocity(-1);
            }
         }
      }
      else
         player->get_ship()->reset_stock();
   }


}

