/*
 *  harbour.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "harbour.h"
#include "good.h"
#include "market.h"
#include "readdata.h"
#include "tools.h"

using namespace std;

Harbour::Harbour(string harbour)
{
   Tools t;
   
   readdata hdata(harbour);
   market = Market(harbour);
   name = hdata.get_str("harbourname");
   crew_available = hdata.get_int("crew_member_available");
   crew_increase = crew_available / 4;
   crew_pay = hdata.get_int("crew_member_pay");
   navpos.longi = hdata.get_float("longitude");
   navpos.lati = hdata.get_float("latitude");
   background = hdata.get_str("background");
   nation = hdata.get_str("nation");
   x11pos.x = hdata.get_int("xpos");
   x11pos.y = hdata.get_int("ypos");

   price_merchant = hdata.get_int("price_merchant");
   price_galleon = hdata.get_int("price_galleon");
   price_brigg = hdata.get_int("price_brigg");
   price_schooner = hdata.get_int("price_schooner");
   price_frigate = hdata.get_int("price_frigate");
   
   governor_avail = false;
   dock_avail = false;

   if (hdata.get_bool("governor"))
      governor_avail = true;

   if (hdata.get_bool("dock"))
      dock_avail = true;
}

Harbour::Harbour(Savegame* sg)
{
   market = Market(sg);
   name = sg->read_str();
   crew_available = sg->read_int();
   crew_increase = sg->read_int();
   crew_pay = sg->read_int();
   navpos = sg->read_navpos();
   background = sg->read_str();
   nation = sg->read_str();
   dock_avail = sg->read_bool();
   governor_avail = sg->read_bool();
   x11pos = sg->read_x11pos();
   price_merchant = sg->read_int();
   price_galleon = sg->read_int();
   price_brigg = sg->read_int();
   price_schooner = sg->read_int();
   price_frigate = sg->read_int();
}

Harbour::Harbour()
{
}

int Harbour::get_crew_available()
{
   return crew_available;
}

navpos_t Harbour::get_position()
{
   return navpos;
}

x11pos_t Harbour::get_x11_position()
{
   return x11pos;
}

int Harbour::get_ship_price(string type)
{
   if (type == "BRIG")
      return price_brigg;
   
   if (type == "GALLEON")
      return price_galleon;

   if (type == "MERCHANT")
      return price_merchant;
   
   if (type == "FRIGATE")
      return price_frigate;

   if (type == "SCHOONER")
      return price_schooner;

   return 0;
}

string Harbour::get_background()
{
   return background;
}

string Harbour::get_nation()
{
   return nation;
}

bool Harbour::has_dock()
{
   return dock_avail;
}

bool Harbour::has_governor()
{
   return governor_avail;
}

Market* Harbour::get_market()
{
   return &market;
}

bool Harbour::get_letter_of_marque(Player* player)
{
   if (player->get_nationality() != get_nation())
      return false;
   else
      return true;
}

void Harbour::turn(World* world)
{
   if (world->is_new_week())
      crew_available += crew_increase;

   market.turn(world);
}


string Harbour::get_name()
{
   return name;
}

int Harbour::get_crew_pay()
{
   return crew_pay;
}

int Harbour::hire_men(int amount)
{
   if ((amount >= 0) && (amount <= crew_available))
   {
      crew_available -= amount;
      return amount;
   }
   else
      return -1;
}

void Harbour::save_to_file(Savegame* sg)
{
   market.save_to_file(sg);
   sg->writeout(name);
   sg->writeout(crew_available);
   sg->writeout(crew_increase);
   sg->writeout(crew_pay);
   sg->writeout(navpos);
   sg->writeout(background);
   sg->writeout(nation);
   sg->writeout(dock_avail);
   sg->writeout(governor_avail);
   sg->writeout(x11pos);
   sg->writeout(price_merchant);
   sg->writeout(price_galleon);
   sg->writeout(price_brigg);
   sg->writeout(price_schooner);
   sg->writeout(price_frigate);
}
















