#ifndef MENU_H
#define MENU_H

#include <vector>
#include <string>
#include <map>
#include <SDL.h>
#include <SDL_ttf.h>

enum menuitem_t
{
   MENU_TYPE_EDIT_STR,
   MENU_TYPE_EDIT_NUM,
   MENU_TYPE_SWITCH,
   MENU_TYPE_INFO,
   MENU_TYPE_SELECT,
   MENU_TYPE_SLIDER,
   MENU_TYPE_IMG
};

class menuselect
{
public:
   menuselect();
   std::string selected;
   std::map<std::string, std::string> strs;
   std::map<std::string, int> ints;
};

class Properties
{
public:
   Properties(SDL_Color on, SDL_Color off, SDL_Color dark, TTF_Font* fnt, SDL_Surface* scr);
   Properties();
   SDL_Color on_color;
   SDL_Color off_color;
   SDL_Color dark_color;
   TTF_Font* font;
   TTF_Font* small_font;
   SDL_Surface* screen;
   std::string background;
};

class Menu_Image
{
public:
   Menu_Image(std::string filename, SDL_Rect pos, Properties properties, bool transparent);
   void render_surface(Properties* p, bool small);
   SDL_Surface* img;
   SDL_Rect item_pos;
};


class Menu_Item
{
public:
   Menu_Item(std::string label, std::vector<std::string> txt, SDL_Rect pos, Properties properties,
             menuitem_t mtype, bool small);
   Menu_Item(std::string label, std::vector<std::string> txt, SDL_Rect pos, Properties properties,
             menuitem_t mtype, int llim, int ulim, bool small);
   Menu_Item(std::string label, std::vector<std::string> txt, SDL_Rect pos, Properties properties,
             menuitem_t mtype, bool active, bool small);
   
   int width();
   int graphical_width(int pos);
   bool is_editable();
   bool is_editable_str();
   bool is_editable_int();
   bool is_switch();
   bool is_selectable();
   bool is_shadowed();
   void render_surface(std::string label, Properties* p, bool small);
   void set_shadowed();
   
   SDL_Rect item_pos;
   std::vector<std::string> text;
   std::string label;
   int subpos;
   std::vector<SDL_Surface*> on;
   std::vector<SDL_Surface*> off;
   std::vector<SDL_Surface*> black;

protected:
   menuitem_t type;
   SDL_Surface* render_surface_on(std::string label, Properties* p, bool small);
   SDL_Surface* render_surface_off(std::string label, Properties* p, bool small);
   bool edit_active;
   bool shadow_input;

};

class Menu
{
public:
   Menu::Menu(Properties* p, int xstart, int ystart, int itemheight, int menuitems);
   Menu::Menu(Properties* p, int menuitems);

   void add_edit_string(std::string txt, std::string defaultstr, int x, int y, bool active);
   void add_edit_string(std::string txt, std::string defaultstr, bool active);
   void add_edit_password(std::string txt, std::string defaultstr, bool active);
   void add_edit_number(std::string txt, int defaultvalue, int x, int y, bool active);
   void add_edit_number(std::string txt, int defaultvalue, bool active);
   void add_label(std::string txt, int x, int y);
   void add_label(int txt, int x, int y);
   void add_slider(std:: string txt, int x, int y, int w, int h, int llim, int ulim);
   void add_switch(std::string label, std::vector<std::string> txt, int x, int y);
   void add_switch(std::string label, std::vector<std::string> txt);
   void add_select_static(std::string txt, int x, int y, bool small);
   void add_select(std::string txt);
   void add_image(std::string filename, int x, int y);
   void add_image_trans(std::string filename, int x, int y);
   void set_active(int pos);
   void redraw(bool cursor, bool onoff, bool static_mode, bool small_mode);
   void recalculate_pos();
   menuselect display();
   menuselect display_static(bool small);

   int get_x();
   int get_y();
   int get_h();
   
protected:
   bool currently_editing;
   bool immediate_return;
   
   SDL_Surface* back;
   SDL_Surface* cursorsurface;
   SDL_Surface* larrowsurface;
   SDL_Surface* rarrowsurface;

   std::vector<Menu_Item> items;
   std::vector<Menu_Item> static_items;
   std::vector<Menu_Item> static_items_small;
   std::vector<Menu_Item> labels;
   std::vector<Menu_Image> images;
   int active;
   Properties properties;
   int x, y, height;
   unsigned int maxitems;
   std::string current_edit;
   
   menuselect select_option(menuselect ret, int selected);
   menuselect select_option_static(menuselect ret, int selected);
   void edit_option(bool cursor_on);
};

#endif























