/*
 *  nation.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <vector>
#include <string>
#include <list>
#include <string>
#include "nation.h"
#include "player.h"

using namespace std;

Playerrank::Playerrank(playerrank_t defaultrank, Player* dplayer)
{
   player = dplayer;
   rank = defaultrank;
   scoring = 0;
   playernationality = dplayer->get_nationality();
}

Playerrank::Playerrank(Savegame* sg, World* world)
{
   player = world->get_player_by_name(sg->read_str());
   rank = (playerrank_t)sg->read_int();
   scoring = sg->read_int();
   playernationality = sg->read_str();
}


playerrank_t Playerrank::get_rank()
{
   return rank;
}

void Playerrank::set_rank(playerrank_t newrank)
{
   rank = newrank;
}

void Nation::set_rank(Player* player, playerrank_t newrank)
{
   for (vector<Playerrank>::iterator pi = playerranks.begin(); pi != playerranks.end(); pi++)
      if (pi->get_player() == player)
         pi->set_rank(newrank);
}

std::string Nation::get_rank_descr(Player* player)
{
   for (vector<Playerrank>::iterator pi = playerranks.begin(); pi != playerranks.end(); pi++)
   {
      if (pi->get_player() == player)
      {
         playerrank_t rnk = pi->get_rank();

         switch(rnk)
         {
         case PS_PIRATE:
            return "Pirate";
            break;
         case PS_FREEBOOTER:
            return "Freebooter";
            break;
         case PS_ENEMY:
            return "Enemy";
            break;
         case PS_NEUTRAL:
            return "Neutral";
            break;
         case PS_ALLIED:
            return "Allied";
            break;
         case PS_COUNT:
            return "Count";
            break;
         case PS_DUKE:
            return "Duke";
            break;
         }
      }
   }
   
   return "undef";
}



Player* Playerrank::get_player()
{
   return player;
}


Nation::Nation(string dnational, playerlist* players)
{
   dutch = NS_PEACE;
   french = NS_PEACE;
   english = NS_PEACE;
   spanish = NS_PEACE;
   nationality = dnational;

   if (dnational == "du")
      dutch = NS_YOURSELF;
   
   if (dnational == "fr")
      french = NS_YOURSELF;

   if (dnational == "en")
      english = NS_YOURSELF;

   if (dnational == "sp")
      spanish = NS_YOURSELF;

   for (playerlist::iterator pi = players->begin(); pi != players->end(); pi++)
   {
      if (pi->get_nationality() == nationality)
         playerranks.push_back(Playerrank(PS_ALLIED, &(*pi)));
      else
         playerranks.push_back(Playerrank(PS_NEUTRAL, &(*pi)));
   }
};

std::string Nation::get_nationality()
{
   return nationality;
}

void Nation::pirate_attack(Player* player)
{
   set_rank(player, PS_PIRATE);
}

void Nation::freebooter_attack(Player* player)
{
   set_rank(player, PS_FREEBOOTER);
}

void Nation::war_attack(Player* player)
{
   set_rank(player, PS_ENEMY);
}

std::string Nation::get_name()
{
   if (nationality == "du")
      return "Netherlands";
   
   if (nationality == "fr")
      return "France";

   if (nationality == "en")
      return "England";

   if (nationality == "sp")
      return "Spain";

   return "undef";
}

playerrank_t Nation::get_rank(Player* player)
{
   for (vector<Playerrank>::iterator pi = playerranks.begin(); pi != playerranks.end(); pi++)
      if (pi->get_player() == player)
         return pi->get_rank();

   // Bogus
   return PS_NEUTRAL;
}


void Nation::turn()
{

}

void Nation::save_to_file(Savegame* sg)
{
   sg->writeout(dutch);
   sg->writeout(french);
   sg->writeout(english);
   sg->writeout(spanish);
   for (vector<Playerrank>::iterator p = playerranks.begin(); p != playerranks.end(); p++)
      p->save_to_file(sg);
   sg->writeout(nationality);
}

Nation::Nation(Savegame* sg, int amount_players, World* world)
{
   dutch = (diplomatic_state_t)sg->read_int();
   french = (diplomatic_state_t)sg->read_int();
   english = (diplomatic_state_t)sg->read_int();
   spanish = (diplomatic_state_t)sg->read_int();

   for (int i=0; i<amount_players; i++)
      playerranks.push_back(Playerrank(sg, world));

   nationality = sg->read_str();
}

void Playerrank::save_to_file(Savegame* sg)
{
   sg->writeout(player->get_name());
   sg->writeout(rank);
   sg->writeout(scoring);
   sg->writeout(playernationality);
}
