/*
 *  nutship.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *  Copyright (C) 2004 Paul Damer
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "nutship.h"
#include <sstream>
#include <string>
#include <cmath>
#include <iostream>

using namespace std;

NutShip::NutShip(vector< vector<string> > filename, int x, int y, int w, int h, int a, int m, int r, int g, int b): 
  Sprite(filename, x, y, w, h, a, m, r, g, b),
  name("unnamed player"),
  r(0),
  g(0),
  b(0),
  floatx(x),
  floaty(y),
  current_boom(4)
{
  rotate_to(a);
}

NutShip::~NutShip()
{
  unload();
}

void NutShip::setcolor(){
  Sprite::setcolor(r,g,b);
}

void NutShip::setcolbyname(string col){
  int rr,rg,rb;
  getcolrgb(col,&rr,&rg,&rb);
  r=rr;
  g=rg;
  b=rb;
  setcolor();
}

void NutShip::getcolrgb(string col, int* gr, int* gg, int* gb){
  if (col=="black"){
    *gr=0;
    *gg=0;
    *gb=0;
  } else if (col=="red"){
    *gr=240;
    *gg=0;
    *gb=0;
  } else if (col=="blue"){
    *gr=0;
    *gg=0;
    *gb=240;
  } else if (col=="pink"){
    *gr=240;
    *gg=0;
    *gb=240;
  } else if (col=="green"){
    *gr=0;
    *gg=240;
    *gb=0;
  } else if (col=="white"){
    *gr=240;
    *gg=240;
    *gb=240;
  } else if (col=="yellow"){
    *gr=240;
    *gg=240;
    *gb=0;
  } else if (col=="grey"){
    *gr=150;
    *gg=150;
    *gb=150;
  } else if (col=="cyan"){
    *gr=0;
    *gg=240;
    *gb=240;
  } else if (col=="brown"){
    *gr=150;
    *gg=0;
    *gb=0;
   }
}

void NutShip::travel(int max_x, int max_y, int windstrength, int winddirection, NutShip* other)
{
   int real_wind = winddirection * 10; 
   int real_angle = -1 * angle - 90;
   int real_course = real_wind - real_angle;

   float real_speed=0;
   float incx;
   float incy;

  while (real_course > 180)
     real_course = real_course - 360;

  int min_wind_angle = abs(real_course);

  if (min_wind_angle <= 35)
  {
     real_speed = 0;
  }

  else if (min_wind_angle > 35 && min_wind_angle <=75) 
  {
     switch(sail_) {
     case 0:
        real_speed=windstrength*0.3;
        break;
     case 1:
        real_speed=windstrength*0.2;
        break;
     case 2:
        real_speed=windstrength*0.1;
        break;
     case 3:
        real_speed=windstrength*0.1;
        break;
     }
  }

  else if (min_wind_angle > 75 && min_wind_angle <=105)
  {
     switch(sail_) {
     case 0:
        real_speed=windstrength*0.1;
        break;
     case 1:
        real_speed=windstrength*0.4;
        break;
     case 2:
        real_speed=windstrength*0.4;
        break;
     case 3:
        real_speed=windstrength*0.2;
        break;
     }
  }
  
  else if (min_wind_angle > 105 && min_wind_angle <=135)
  {
     switch(sail_) {
     case 0:
        real_speed=windstrength*0.2;
        break;
     case 1:
        real_speed=windstrength*0.3;
        break;
     case 2:
        real_speed=windstrength*0.5;
        break;
     case 3:
        real_speed=windstrength*0.6;
        break;
     }
  }
    
  else if (min_wind_angle > 135 && min_wind_angle <=180)
  {
     switch(sail_) {
     case 0:
        real_speed=windstrength*0.1;
        break;
     case 1:
        real_speed=windstrength*0.2;
        break;
     case 2:
        real_speed=windstrength*0.3;
        break;
     case 3:
        real_speed=windstrength*0.4;
        break;
     }
     
  }

  
  int canvas;

  switch(sail_) {
  case 0:
     canvas = 4;
     break;
  case 1:
     if (real_course <0)
        canvas = 5;
     else if (real_course >=0)
        canvas = 3;
     break;
  case 2:
     if (real_course < 0)
        canvas = 6;
     else if (real_course >=0)
        canvas = 2;
     break;
  case 3:
     if (real_course < 0)
        canvas = 7;
     else if (real_course >= 0)
        canvas = 1;
     break;
  default:
    canvas=4;
  }
 
  if (canvas > current_boom)
  {
          current_boom++;
  }
  else if (canvas < current_boom)
  {
          current_boom--;
  }

  real_speed = coasting_speed*.8 + real_speed*.2;
  coasting_speed = real_speed;
  
  incx = real_speed*cos((float)angle/180*PI);
  incy = real_speed*sin((float)angle/180*PI);

  float newx=floatx;
  float newy=floaty;

  floatx=newx+incx;
  floaty=newy+incy;

  // Collision detection for the borders
  if (rint(floatx)<0) floatx=0;
  if (rint(floaty)<0) floaty=0;
  if ((rint(floatx)+rect.w)>max_x) floatx=max_x-rect.w;
  if ((rint(floaty)+rect.h)>max_y) floaty=max_y-rect.h;

  Sprite::move_to(static_cast<int>(rint(floatx)),static_cast<int>(rint(floaty)));
  Sprite::rotate_baum(current_boom-1);
//  cout << real_wind << " "<< angle<< " " << real_angle << " " << real_course << " " << real_speed << "\n";

}

void NutShip::canvas_modify(int v){
  sail_=sail_+v;
  if (sail_<0) sail_=0;
  if (sail_>3) sail_=3;
}

bool NutShip::collides(int groesse, int x, int y)
{
   if (((floatx >= (x-groesse)) && (floatx <= (x+groesse))) &&
       ((floaty >= (y-groesse)) && (floaty <= (y+groesse))))
      return true;
   else
      return false;
}

void NutShip::speed_modify(int v){
  speed_=speed_+v;
  if (speed_<-3) speed_=-3;
  if (speed_>3) speed_=3;
}

void NutShip::move(int x, int y)
{
  floatx+=x;
  floaty+=y;
  Sprite::move(x,y);
}

void NutShip::move_to(int x, int y)
{
  floatx=x;
  floaty=y;
  Sprite::move(x,y);
}

int NutShip::get_x()
{
   return (int)floatx;
}

int NutShip::get_y()
{
   return (int)floaty;
}

int NutShip::get_a()
{
   return Sprite::angle;
}

int NutShip::speed() const
{
   return speed_;
}















