/*
 *  player.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "player.h"
#include "harbour.h"
#include "ship.h"
#include "tools.h"
#include "savegame.h"
#include <string>
#include <sstream>

using namespace std;

Player::Player(string pname, string pnation, std::string pdifficulty, Harbour* start_harbour, string
               advantage, string pcolor)
{
   Tools t;
   gameplay_parameters = readdata(t.data_path("gameplay-parameters.conf"));

   string shipname = "undefined";
   name = pname;
   nationality = pnation;
   difficulty = pdifficulty;
   harbour_pos = start_harbour;
   navigation_skill = 0;
   color=pcolor;
   charme_skill = 0;
   leadership_skill = 0;
   gunner_skill = 0;
   ferocity = 0;
   gold = 0;
   ship = Ship("NONE", shipname, 50);
   player_state = "PS_HARBOUR";
   remaining_distance = 0;
   target_harbour = start_harbour;
   artificial_intelligence = false;
   player_event = GE_NONE;
   pirate_points = 0;
   dead = false;
   freebooting = false;
   
   // FIXME, the differentation of the gameplay details will be done later on
   // Right now everything is medium

   if (difficulty == "DIFF_EASY")
      gold = gameplay_parameters.get_int("initial_gold");
   
   if (difficulty == "DIFF_MEDIUM")
      gold = gameplay_parameters.get_int("initial_gold");

   if (difficulty == "DIFF_HARD")
      gold = gameplay_parameters.get_int("initial_gold");

   if (advantage == "Charme")
      raise_charme(1);
   if (advantage == "Navigation")
      raise_navigation(1);
   if (advantage == "Gunning")
      raise_gunner(1);
   if (advantage == "Leadership")
      raise_leadership(1);
}

Player::Player(Savegame* sg, World* world)
{
   Tools t;
   name= sg->read_str();
   color = sg->read_str();
   difficulty = sg->read_str();
   player_state = sg->read_str();
   // harbour_pos
   harbour_pos = world->get_harbour_by_name(sg->read_str());
   ferocity = sg->read_int();
   navigation_skill = sg->read_int();
   charme_skill = sg->read_int();
   leadership_skill = sg->read_int();
   gunner_skill = sg->read_int();
   gold = sg->read_int();
   ship = Ship(sg);
   remaining_distance = sg->read_float();
   // target_harbour
   target_harbour = world->get_harbour_by_name(sg->read_str());
   nationality = sg->read_str();
   artificial_intelligence = sg->read_bool();
   player_event = (game_event_t)sg->read_int();
   pirate_points = sg->read_int();
   freebooting = sg->read_bool();
   dead = sg->read_bool();
   gameplay_parameters = readdata(t.data_path("gameplay-parameters.conf"));
}

Player::Player()
{
   
}

void Player::kill()
{
   dead = true;
}

bool Player::is_dead()
{
   return dead;
}

bool Player::is_ai()
{
   return artificial_intelligence;
}

void Player::set_ai(bool a)
{
   artificial_intelligence=a;
}

void Player::save_to_file(Savegame* sg)
{
   sg->writeout(name);
   sg->writeout(color);
   sg->writeout(difficulty);
   sg->writeout(player_state);
   sg->writeout(harbour_pos);
   sg->writeout(ferocity);
   sg->writeout(navigation_skill);
   sg->writeout(charme_skill);
   sg->writeout(leadership_skill);
   sg->writeout(gunner_skill);
   sg->writeout(gold);
   ship.save_to_file(sg);
   sg->writeout(remaining_distance);
   sg->writeout(target_harbour);
   sg->writeout(nationality);
   sg->writeout(artificial_intelligence);
   sg->writeout(player_event);
   sg->writeout(pirate_points);
   sg->writeout(freebooting);
   sg->writeout(dead);
}

void Player::add_points(string beaten)
{
   if (beaten == "MERCHANT")
      pirate_points += gameplay_parameters.get_int("pirate_points_merchant");
      
   if (beaten == "GALLEON")
      pirate_points += gameplay_parameters.get_int("pirate_points_galleon");

   if (beaten == "SCHOONER")
      pirate_points += gameplay_parameters.get_int("pirate_points_schooner");

   if (beaten == "BRIG")
      pirate_points += gameplay_parameters.get_int("pirate_points_brig");
   
   if (beaten == "FRIGATE")
      pirate_points += gameplay_parameters.get_int("pirate_points_frigate");
}

void Player::set_round(int pturn)
{
   Player::gameturn = pturn;
}

int Player::get_round()
{
   return gameturn;
}

float Player::get_remaining_distance()
{
   return remaining_distance;
}

void Player::set_freebooting(bool setfreebooting)
{
   freebooting = setfreebooting;
}

bool Player::is_freebooter()
{
   return freebooting;
}


string Player::get_difficulty()
{
   return difficulty;
}

bool Player::board_ship(Player* opponent)
{
   float own_men = get_ship()->get_crew_members();
   float opp_men = opponent->get_ship()->get_crew_members();
   
   if (get_leadership() == 1)
      own_men *= gameplay_parameters.get_float("seabattle_leadership_bonus_good");
   if (get_leadership() == 2)
      own_men *= gameplay_parameters.get_float("seabattle_leadership_bonus_excellent");
   
   if (opponent->get_leadership() == 1)
      opp_men *= gameplay_parameters.get_float("seabattle_leadership_bonus_good");
   if (opponent->get_leadership() == 2)
      opp_men *= gameplay_parameters.get_float("seabattle_leadership_bonus_excellent");

   int rndfactor = rand() % 60;
   float factor1 = (rndfactor += 70) / 100;
   
   rndfactor = rand() % 60;
   float factor2 = (rndfactor += 70) / 100;
   
   if ((own_men*factor1) > (opp_men*factor2))
      return true;

   return false;
}

string Player::get_state()
{
   return player_state;
}

game_event_t Player::get_event()
{
   return player_event;
}

Harbour* Player::get_harbour_pos()
{
   return harbour_pos;
}

Harbour* Player::get_target_harbour()
{
   return target_harbour;
}

bool Player::has_ship()
{
   if (get_ship()->get_type() == "NONE")
      return false;
   
   return true;
}

void Player::set_ship(string stype, std::string sname, int sstate)
{
   ship = Ship(stype, sname, sstate);
}

void Player::turn(World* world)
{
   Tools t;

   if (player_state == "PS_ONSEA")
   {
      if (navigation_skill == 2)
         remaining_distance -= (gameplay_parameters.get_float("seatravel_navigation_bonus_good")
                                *world->compute_travel_speed(&ship));

      if (navigation_skill == 1)
         remaining_distance -= (gameplay_parameters.get_float("seatravel_navigation_bonus_excellent")
                                *world->compute_travel_speed(&ship));

      if (navigation_skill == 0)
         remaining_distance -= world->compute_travel_speed(&ship);


      if (t.rnd_percent(gameplay_parameters.get_int("probability_for_ship_encounter")))
      {
         player_event = GE_SEABATTLE;
      }

      if (t.rnd_percent(gameplay_parameters.get_int("probability_for_scurvy")))
      {
         player_event = GE_SCURVY;
      }

      if (t.rnd_percent(gameplay_parameters.get_int("probability_for_a_stormy_day")))
      {
         player_event = GE_STORM;
      }

      if (remaining_distance < 0)
      {
         player_state = "PS_HARBOUR";
         harbour_pos = target_harbour;
         target_harbour = 0;
      }

      // FIXME
      // For now we don't hassle the player with additional events if he's
      // already on sea with a starving crew. In upcoming version we should
      // build a stack of events and solve this cleanly
      if (get_ship()->is_starving())
         player_event = GE_STARVE;
   }

   if (world->is_new_week())
   {
      if (get_ship()->get_crew_members() > 0)
      {
         int payed_gold = (get_ship()->pay_crew(get_ship()->get_pay_percent(), get_gold()));

         if (payed_gold != -1)
            add_gold(-payed_gold);
         else
            player_event = GE_MUTINY;
      }
   }

   ship.turn(world->is_new_week());
}

void Player::set_state(string new_state)
{
   player_state = new_state;
}

void Player::set_event(game_event_t new_event)
{
   player_event = new_event;
}

void Player::stay_in_harbour()
{
   get_ship()->remain_in_harbour();
}

bool Player::opponent_could_escape(Player* player, World* world)
{
   int rndfactor = rand() % 60;
   float factor1 = (rndfactor += 70) / 100;
   
   rndfactor = rand() % 60;
   float factor2 = (rndfactor += 70) / 100;
   
   if (((get_ship()->get_speed())*factor1) < ((player->get_ship()->get_speed())*factor2))
      return true;
   
   return false;
}

bool Player::escape_from_enemy(Player* player, World* world)
{
   int rndfactor = rand() % 60;
   float factor1 = (rndfactor += 70) / 100;
   
   rndfactor = rand() % 60;
   float factor2 = (rndfactor += 70) / 100;
   
   if (get_ship()->get_speed()*factor1 > player->get_ship()->get_speed()*factor2)
      return true;
   
   return false;
}


bool Player::opponent_is_willing_to_fight(Player* opponent, World* world)
{
   string opponent_type = opponent->get_ship()->get_type();
   string nation_encounter = opponent->get_nationality();

   if (opponent_type == "MERCHANT" || opponent_type == "GALLEON")
      return false;

   if (opponent->get_nationality() == "pi")
      return opponent->get_ship()->has_better_or_equal_shiptype(&ship);
   else
   {
      if (opponent_type == "BRIG" || opponent_type == "SCHOONER" || opponent_type == "FRIGATE")
      {
         playerrank_t rankenc= world->get_nation(nation_encounter)->get_rank(this);
         
         if (rankenc == PS_PIRATE || rankenc == PS_FREEBOOTER || rankenc == PS_ENEMY)
            return opponent->get_ship()->has_better_or_equal_shiptype(&ship);
      }
   }
   
   return false;
}

void Player::set_target_harbour()
{
   harbour_pos = target_harbour;
}

void Player::raise_navigation(int p)
{
   navigation_skill += p;
}

void Player::raise_charme(int p)
{
   charme_skill += p;
}

void Player::raise_leadership(int p)
{
   leadership_skill += p;
}

void Player::raise_gunner(int p)
{
   gunner_skill += p;
}

void Player::raise_ferocity(int p)
{
   ferocity += p;
}

int Player::get_navigation()
{
   return navigation_skill;
}

int Player::get_points()
{
   return pirate_points;
}


int Player::get_charme()
{
   return charme_skill;
}

int Player::get_leadership()
{
   return leadership_skill;
}

int Player::get_gunner()
{
   return gunner_skill;
}

string Player::get_navigation_descr()
{
   switch(navigation_skill)
   {
   case 0:
      return "Average";
   case 1:
      return "Good";
   case 2:
      return "Excellent";
   }

   return "undef";
}

string Player::get_charme_descr()
{
   switch(charme_skill)
   {
   case 0:
      return "Average";
   case 1:
      return "Good";
   case 2:
      return "Excellent";
   }

   return "undef";
}

string Player::get_leadership_descr()
{
   switch(leadership_skill)
   {
   case 0:
      return "Average";
   case 1:
      return "Good";
   case 2:
      return "Excellent";
   }

   return "undef";
}

string Player::get_gunner_descr()
{
   switch(gunner_skill)
   {
   case 0:
      return "Average";
   case 1:
      return "Good";
   case 2:
      return "Excellent";
   }

   return "undef";
}

int Player::get_gold()
{
   return gold;
}

int Player::get_ferocity()
{
   return ferocity;
}

string Player::get_ferocity_descr()
{

   if (ferocity == 0)
      return "Average";
   
   if (ferocity < 0)
      return "Gallantly";

   if (ferocity > 0)
      return "Gruesome";

   return "undef";
}

string Player::get_name()
{
   return name;
}

string Player::get_col()
{
   return color;
}

void Player::set_col(string c)
{
   color=c;
}

void Player::add_gold(int p)
{
   gold += p;
}

Ship* Player::get_ship()
{
   return &ship;
}

void Player::sail_to(Harbour* target, World* world)
{
   remaining_distance = world->compute_harbour_distance(harbour_pos, target);
   target_harbour = target;
}

string Player::get_position_description()
{
   if (player_state == "PS_ONSEA")
   {
      ostringstream ost;
      ost << "On sea (" << remaining_distance << " sm to go)";
      return ost.str();
   }
   if (player_state == "PS_HARBOUR" || get_remaining_distance() < 0)
   {
      if (has_ship())
         return "Anchored at " + get_harbour_pos()->get_name();
      else
         return "Staying at " + get_harbour_pos()->get_name();
   }
   return "Undefined";
}

std::string Player::get_nationality()
{
   return nationality;
}














