#ifndef PLAYER_H
#define PLAYER_H
#include "ship.h"
#include "world.h"
#include <string>
#include "nation.h"
#include "harbour.h"
#include "savegame.h"

class Harbour;
class World;
class Playerrank;
class Savegame;

enum difficulty_t
{
   DIFF_EASY,
   DIFF_MEDIUM,
   DIFF_HARD,
   DIFF_AI
};

enum game_event_t
{
   GE_NONE,
   GE_SCURVY,
   GE_MUTINY,
   GE_SEABATTLE,
   GE_STORM,
   GE_STARVE
};

class Player
{
public:
   Player(std::string pname, std::string nation, std::string difficulty, Harbour* start_harbour, std::string advantage,
          std::string pcolor);
   Player(Savegame* sg, World* world);
   Player();
   std::string get_state();
   void set_state(std::string new_state);
   void set_event(game_event_t new_event);
   game_event_t get_event();
   Harbour* get_harbour_pos();
   Harbour* get_target_harbour();
   void turn(World* world);
   void raise_navigation(int p);
   void raise_charme(int p);
   void raise_leadership(int p);
   void raise_ferocity(int);
   void raise_gunner(int p);
   void add_points(std::string beaten);
   int get_navigation();
   int get_charme();
   std::string get_difficulty();
   int get_ferocity();
   int get_leadership();
   int get_gunner();
   int get_points();
   std::string get_navigation_descr();
   std::string get_charme_descr();
   std::string get_ferocity_descr();
   std::string get_leadership_descr();
   std::string get_gunner_descr();
   int get_gold();
   void set_freebooting(bool setfreebooting);
   void set_ship(std::string stype, std::string sname, int sstate);
   bool opponent_is_willing_to_fight(Player* player, World* world);
   bool opponent_could_escape(Player* player, World* world);
   bool escape_from_enemy(Player* player, World* world);
   Ship* get_ship();
   std::string get_name();
   std::string get_col();
   void set_col(std::string);
   void add_gold(int);
   std::string get_position_description();
   void sail_to(Harbour* target, World* world);
   bool board_ship(Player* opponent);

   void set_round(int pturn);
   int get_round();
   
   void stay_in_harbour();
   float get_remaining_distance();
   void set_target_harbour();
   bool has_ship();
   bool is_freebooter();
   void kill();
   bool is_dead();
   bool is_ai();
   void set_ai(bool);
   std::string get_nationality();
   std::string get_rank(std::string nation, World* world);

   void Player::save_to_file(Savegame* sg);

protected:
   readdata gameplay_parameters;

   std::string name;
   std::string color;
   std::string difficulty;
   std::string player_state;
   Harbour* harbour_pos;
   int ferocity;
   int navigation_skill;
   int charme_skill;
   int leadership_skill;
   int gunner_skill;
   int gold;
   Ship ship;
   float remaining_distance;
   Harbour* target_harbour;
   std::string nationality;
   bool artificial_intelligence;
   game_event_t player_event;
   int pirate_points;
   bool freebooting;
   bool dead;
   int gameturn;
};


#endif
















