/*
 *  readdata.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <fstream>
#include <string>
#include "readdata.h"
#include <stdio.h>
#include <utility>
#include <cstdlib>

using namespace std;

readdata::readdata(string dataset)
{
   parse_file_concrete(dataset);
}

std::string readdata::get_str(std::string data)
{
   if (datalist[data] == "")
   {
      cerr << "Could not read parameter " << data << endl;
      exit(-1);
   }

   return datalist[data];
};

std::string readdata::get_str_blanked(std::string data)
{
   if (datalist[data] == "")
   {
      cerr << "Could not read parameter " << data << endl;
      exit(-1);
   }
   return rem_blanks(datalist[data]);
};

int readdata::get_int(std::string data)
{
   if (datalist[data] == "")
   {
      cerr << "Could not read parameter " << data << endl;
      exit(-1);
   }

   return atoi(datalist[data].c_str());
};

float readdata::get_float(std::string data)
{
   if (datalist[data] == "")
   {
      cerr << "Could not read parameter " << data << endl;
      exit(-1);
   }

   return atof(datalist[data].c_str());
};

bool readdata::get_bool(std::string data)
{
   bool ret = false;

   if (datalist[data] == "")
   {
      cerr << "Could not read parameter " << data << endl;
      exit(-1);
   }
   
   if (datalist[data] == "true")
      ret = true;
   else
      ret = false;
   
   return ret;
};


void readdata::parse_file_concrete(string filepath)
{
   ifstream data_file_stream;
   string dataline;
   map<string, string> options;
   unsigned int hashmark, delimiter;
   string left, right;

   data_file_stream.open(filepath.c_str(), ios_base::in);
   while (getline(data_file_stream, dataline)) {

      // weed out comments
      hashmark = dataline.find('#');
      if (hashmark != std::string::npos)
         dataline.erase(hashmark, dataline.length()-hashmark);

      // split value/key pair
      delimiter = dataline.find('=');
      if (delimiter != string::npos) {
         left = rem_blanks(dataline.substr(0, delimiter));
         right = dataline.substr(delimiter+1, dataline.length());
         datalist.insert(make_pair(left, right));

      };
   };
   data_file_stream.close();
   
};

readdata::readdata()
{
}

string readdata::rem_blanks(string str)
{
   string shortened;
      
   for (unsigned int i=0; i < str.length(); i++)
      if (str[i] != ' ')
         shortened += str[i];
      
   return shortened;
};
















