/*
 *  savegame.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <string>
#include <cstdlib>
#include "savegame.h"
#include "harbour.h"
#include "player.h"
#include "world.h"
#include <sys/stat.h>
#include <sys/types.h>

using namespace std;

Savegame::Savegame(std::string filename, bool read)
{
   Savegame::savegamename = "";

   if (read)
      readfile.open(filename.c_str(), ios_base::in);
   else
   {
      writefile.open(filename.c_str(), ios_base::out);
      Tools t;
#ifndef WIN32
      mkdir(t.savegame_path("").c_str(), 0777);
#endif   
   }
}


Savegame::Savegame()
{
   writefile.open("/home/jmm/foobar", ios_base::out);
}

Savegame::~Savegame()
{
   writefile.close();
}

void Savegame::write_game(std::vector<Player> *players, std::vector<Harbour> *harbours,
                          std::vector<Nation> *nations, World* world)
{
   writefile << harbours->size() << endl;
   writefile << players->size() << endl;
   writefile << nations->size() << endl;

   for (vector<Harbour>::iterator h = harbours->begin(); h != harbours->end(); h++)
      h->save_to_file(this);

   writefile << "END-H BEG-P" << endl;

   for (vector<Player>::iterator p = players->begin(); p != players->end(); p++)
      p->save_to_file(this);

   writefile << "END-P BEG-N" << endl;

   for (vector<Nation>::iterator n = nations->begin(); n != nations->end(); n++)
      n->save_to_file(this);

   writefile << "END-N" << endl;

}

int Savegame::get_turn()
{
   return turn;
}

vector<Harbour> Savegame::get_harbours()
{
   vector<Harbour> hrb;
   for (int i=0; i<amount_of_harbours; i++)
      hrb.push_back(Harbour(this));

   return hrb;
}

vector<Player> Savegame::get_players(World* world)
{
   vector<Player> pls;
   for (int i=0; i<amount_of_players; i++)
      pls.push_back(Player(this, world));

   return pls;
}

vector<Nation> Savegame::get_nations(World* world)
{
   vector<Nation> nats;
   for (int i=0; i<amount_of_nations; i++)
      nats.push_back(Nation(this, amount_of_players, world));

   return nats;
}


void Savegame::writeout(int d)
{
   writefile << d << endl;
}

void Savegame::writeout(string d)
{
   writefile << d << endl;
}

void Savegame::writeout(navpos_t d)
{
   writefile << d.longi << endl;
   writefile << d.lati << endl;
}

void Savegame::writeout(x11pos_t d)
{
   writefile << d.x << endl;
   writefile << d.y << endl;
}

void Savegame::writeout(bool d)
{
   if (d)
      writefile << "true" << endl;
   else
      writefile << "false" << endl;
}

void Savegame::writeout(float d)
{
   writefile << d << endl;
}

void Savegame::writeout(Harbour* d)
{
   writefile << d->get_name() << endl;
}

void Savegame::write_header(int turn, string savegamename)
{
   writefile << "Freebooters Savegame" << endl;
   writefile << "Version 0.2.1" << endl;
   writefile << savegamename << endl;
   writefile << turn << endl;
}

bool Savegame::read_header()
{
   string s;

   if (!readfile.good())
   {
      return false;
   }

   getline(readfile, s);
   if (s != "Freebooters Savegame")
   {
      cerr << "This is no Freebooters savegame" << endl;
      return false;
   }

   getline(readfile, s);
   if (s != "Version 0.2.1")
   {
      cerr << "Incompatible Freebooters savegame" << endl;
      return false;
   }

   savegamename = read_str();
   turn = read_int();

   amount_of_harbours = read_int();
   amount_of_players = read_int();
   amount_of_nations = read_int();
   
   return true;
}

int Savegame::read_int()
{
   string s;
   getline(readfile, s);

   return atoi(s.c_str());
}

bool Savegame::read_bool()
{
   string s;
   getline(readfile, s);
   if (s == "true")
      return true;
   
   return false;
}

string Savegame::read_str()
{
   string s;
   getline(readfile, s);

   return s;
}

bool Savegame::read_check(string checkpoint)
{
   string s;
   getline(readfile, s);

   if (s == checkpoint)
      return true;

   return false;
}

float Savegame::read_float()
{
   string s;
   getline(readfile, s);

   return (float)atof(s.c_str());
}

navpos_t Savegame::read_navpos()
{
   string s;
   navpos_t n;
   getline(readfile, s);
   n.longi = atof(s.c_str());

   getline(readfile, s);
   n.lati = atof(s.c_str());
   
   return n;
}

x11pos_t Savegame::read_x11pos()
{
   string s;
   x11pos_t x;
   getline(readfile, s);
   x.x = atoi(s.c_str());

   getline(readfile, s);
   x.y = atoi(s.c_str());
   
   return x;
}

string Savegame::get_name()
{
   return savegamename;
}








