/*
 *  sdl_setup.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL_ttf.h>
#include <SDL_image.h>
#include "sdl_setup.h"
#include <sys/time.h>
#include <cstdlib>
#include <iostream>
#include "readdata.h"

using namespace std;

SDL_Surface* SDL_Setup::sdl()
{
   srand(time(NULL));
   SDL_Surface* screen;

   if(SDL_Init(SDL_INIT_VIDEO| SDL_INIT_AUDIO|SDL_INIT_TIMER) == -1)
   {
      cerr << "Can't init SDL: " << SDL_GetError() << endl;
      exit(1);
   }

   atexit(SDL_Quit); 

   screen = SDL_SetVideoMode(640, 480, 16, SDL_HWSURFACE | SDL_DOUBLEBUF);
   if(screen == NULL)
   {
      cerr << "Can't set video mode: " << SDL_GetError() << endl;
      exit(1);
   }
   SDL_WM_SetCaption("Freebooters", 0);
   
   int audio_rate = 22050;
   Uint16 audio_format = AUDIO_S16; 
   int audio_channels = 2;
   int audio_buffers = 4096;


   readdata config("/etc/freebooters.conf");

   if (config.get_str("sound") != "no")
   {
      if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers))
      {
         cerr << "Initialization of audio device failed: " << Mix_GetError() << endl;
//         exit(1);
      }
   }

   return screen;
}

void SDL_Setup::cleanup()
{
   Mix_CloseAudio();
   SDL_Quit();
   exit(0);
}












