/*
 *  shots.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "tools.h"
#include "shots.h"
#include <sstream>

#define PI 3.14159265

using namespace std;

Shots::Shots()
{
   Tools t;
   balls.push_back(t.load_img(t.sprites_path("ball1.png"))); // this one does not get colorkeyed!?
   balls.push_back(t.load_img(t.sprites_path("ball2.png")));
   balls.push_back(t.load_img(t.sprites_path("ball3.png")));
   balls.push_back(t.load_img(t.sprites_path("ball4.png")));
   balls.push_back(t.load_img(t.sprites_path("ball5.png")));
   balls.push_back(t.load_img(t.sprites_path("ball6.png")));
   balls.push_back(t.load_img(t.sprites_path("ball7.png")));
   balls.push_back(t.load_img(t.sprites_path("ball8.png")));
   
   Splash splashes();

   for (vector<SDL_Surface*>::iterator i =balls.begin(); i != balls.end (); i++){
      SDL_SetColorKey (*i,SDL_SRCCOLORKEY,SDL_MapRGB((*i)->format,0,0,255));
   }
}

Shots::~Shots(){
}

void Shots::unload(){
  for (vector<SDL_Surface*>:: iterator b = balls.begin(); b != balls.end(); b++) {
    SDL_FreeSurface(*b);
  }
  balls.clear();
  shots.clear();
}

void Shots::clear(){
  shots.clear();
}

void Shots::drawShots(SDL_Surface* target){

  for (vector<shot>::iterator i =shots.begin(); i != shots.end (); i++){
    i->airtime++;

    // calculate cannonball flight altitude
    if (i->airtime<=(i->firepower/2)){
      i->h=static_cast<int>(rint((float)i->airtime/(float)i->firepower*2.0*8.0));
    } else {
      i->h=static_cast<int>(rint(8.0-((((float)i->airtime/(float)i->firepower*2.0)-1.0)*8.0)));
    }

    // calculate cannonball position
    i->x=static_cast<int>(rint((float)i->airtime*10.0*cos(i->origin_a*PI/180.0)))+i->origin_x+13;
    i->y=static_cast<int>(rint((float)i->airtime*10.0*sin(i->origin_a*PI/180.0)))+i->origin_y+13;

    if(i->x<639 && i->y<479 && i->x>0 && i->y>0 && i->h<1) {
      splashes.addSplash(i->x,i->y);
    }
 
    if(i->x>639 || i->y>479 || i->x<0 || i->y<0 || i->h<1) {
      i=shots.erase(i);
      i--;
      break;
    }

    rect.x=i->x;
    rect.y=i->y;

    SDL_BlitSurface(balls[i->h-1], 0, target, &rect);
  }
  splashes.drawSplashes(target);
}

void Shots::addShot(Combatant* c)
{
   Tools t;
   t.play_sound(c->shot);
   shot current;
   current.x=c->ship->get_x()+10;
   current.y=c->ship->get_y()+10;
   current.h=1;
   current.origin_x=current.x;
   current.origin_y=current.y;
   // cannon is located larboard
   current.origin_a=c->ship->get_a()+270;
   if (current.origin_a>360) current.origin_a-=360;
   current.firepower=c->firepower;
   current.airtime=1;
   shots.push_back(current);
   c->firepower=0;
   c->lastshot_time=SDL_GetTicks();
   c->player->get_ship()->decrease_ammo();
}

bool Shots::isDead(Combatant* c){
  for (vector<shot>::iterator it =shots.begin(); it != shots.end (); it++) {
    if ( ((it->x >= c->ship->get_x() && (it->x <= c->ship->get_x()+70)) 
	  && ((it->y >=  c->ship->get_y()) && (it->y <= c->ship->get_y()+70)) && (it->h < 2)) 
	 && it->airtime>it->firepower/2) {
      it=shots.erase(it);
      it--;
      c->player->get_ship()->add_damage();
      c->explosion_frame=1; // blow it up!
    }
  }
  return c->player->get_ship()->is_sinking();
}








