/*
 *  tools.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "tools.h"
#include <string>
#include <iostream>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <SDL_mixer.h>
#include "readdata.h"
#include <sys/types.h>

#ifndef WIN32
#include <pwd.h>
#endif


#include <unistd.h>

using namespace std;

Tools::Tools()
{
#ifdef WIN32
   enable_sound = true;
   path_to_data_dir = "";
   return;
#else
   readdata config("/etc/freebooters.conf");

   if (config.get_str("datadir") == "")
      path_to_data_dir = "";
   else
      path_to_data_dir = config.get_str("datadir");

   if (config.get_str("sound") == "no")
      enable_sound = false;
   else
      enable_sound = true;
#endif
}

SDL_Surface* Tools::load_img(string filename)
{
   SDL_Surface* temp = IMG_Load(filename.c_str());
   if ( temp == 0 )
   {
      cerr << "Could not load image: " << filename << endl;
      return 0;
   }
   else
   {
      SDL_Surface* image = SDL_DisplayFormat(temp);
      SDL_FreeSurface(temp);
      return image;
   }
}

SDL_Surface* Tools::load_img_alpha(string filename)
{
   SDL_Surface* im = load_img(filename);

   if ( im != 0 )
   {
      SDL_SetAlpha(im, SDL_SRCALPHA, 128);
      return im;
   }

   return 0;
}


TTF_Font* Tools::load_ttf(int size, string fontname)
{
   TTF_Font* menu_font;

   if ( TTF_Init() == -1 )
   {
      cerr << "TTF: " << TTF_GetError() << endl;
      exit(2);
   }

   menu_font = TTF_OpenFont(fonts_path(fontname).c_str(), size);
   if (! menu_font )
   {
      cerr << "Could not load font" << endl;
      exit(2);
   }
   SDL_ShowCursor(SDL_ENABLE);
   return menu_font;
}

Mix_Chunk* Tools::load_wav(string wavname)
{
   Mix_Chunk* wave = Mix_LoadWAV(sound_path(wavname).c_str());

   if(!wave)
      cerr << "Could not load sound :" << Mix_GetError() << endl;

   return wave;
}

Mix_Music* Tools::load_mus(string musname)
{
   Mix_Music* mus = Mix_LoadMUS(sound_path(musname).c_str());

   if(!mus)
      cerr << "Could not load sound :" << Mix_GetError() << endl;

   return mus;
}

string Tools::fonts_path(std::string fontname)
{ 
   return path_to_data_dir + fontname;
}

string Tools::rnd_list(std::vector<std::string> options)
{
   int r = rand() % options.size();
   return options.at(r);
}

string Tools::data_path(std::string data)
{
   return path_to_data_dir + data;
}

string Tools::sound_path(std::string sound)
{
   return path_to_data_dir + "sound/" + sound;
}

string Tools::sprites_path(std::string picture)
{
   return path_to_data_dir + "sprites/" + picture;
}

string Tools::harbour_data_path(string harbour)
{
   return path_to_data_dir + "harbours/" + harbour;
}

string Tools::graphics_path(string picture)
{
   return path_to_data_dir + "graphics/" + picture;
}

void Tools::play_sound (Mix_Chunk* snd)
{
   if (Tools::enable_sound)
      Mix_PlayChannel(-1, snd, 0);
}

bool Tools::rnd_percent(int probability)
{
   int r = rand() % 100;
   if (probability < 0)
      probability = 0;

   if (probability > 100)
      probability = 100;

   if (r < probability)
      return true;

   return false;
}

string Tools::random_col(){
  switch(rand()%9){
  case 0:
    return "black";
  case 1:
    return "red";
  case 2:
    return "blue";
  case 3:
    return "pink";
  case 4:
    return "green";
  case 5:
    return "white";
  case 6:
    return "yellow";
  case 7:
    return "cyan";
  case 8:
    return "brown";
  }
  return "black";
}

string Tools::savegame_path(string savegame)
{
  string savegamepath;
#ifdef WIN32
    savegamepath = savegame;
#else
   savegamepath = "/home/";
   savegamepath += getpwuid(getuid())->pw_name;
   savegamepath += "/.freebooters/" + savegame;
#endif
  return savegamepath;
}

