#ifndef WORLD_H
#define WORLD_H

#include "harbour.h"
#include "market.h"
#include "player.h"
#include "savegame.h"
#include "readdata.h"
#include <vector>
#include <string>
#include <list>
#include <string>

class Harbour;
class Player;
class Ship;
class Nation;
class Good;
struct savegame_description;

enum game_state_t
{
   GS_STARTMENU,
   GS_RUNNING,
   GS_GAMEOVER
};

typedef std::vector<Harbour> harbourlist;
typedef std::vector<Player> playerlist;
typedef std::vector<Good> goodslist;
typedef std::vector<Nation> nationlist;

class World
{
public:
   World();
   void add_player(Player player);
   void add_harbour(std::string harbourname);
   void add_nation(std::string nationality);

   Player* get_player(int index);
   Player* get_opponent(int index);
   harbourlist* get_harbours();
   playerlist* get_players();
   nationlist* get_nations();

   void remove_players();

   bool new_goods_produced();
   void create_opponent();
   void remove_opponent(int index);
   int get_turn();
   float compute_harbour_distance(Harbour* from, Harbour* to);
   float compute_travel_speed (Ship* ship);

   Harbour* get_harbour_by_name(std::string harbourname);
   Player* get_player_by_name(std::string playername);

   int get_nr_harbours();
   int get_nr_nations();
   int get_nr_players();

   void set_state(game_state_t gstate);
   game_state_t get_state();

   void turn();
   bool is_new_week();

   void save_game(std::string filename, std::string _savegamename);
   void load_game(std::string filename, std::string savegamename);
   std::vector<savegame_description> get_savegame_names();
   
   Nation* get_nation(std::string nationality);

protected:
   playerlist players;
   playerlist opponents;
   std::vector<Harbour> harbours;
   int gturn;
   game_state_t game_state;
   std::vector<Nation> nations;
   std::string difficulty_level;

   readdata gameplay;
};

#endif










